@@ -55,6 +55,7 @@ from orca_python.exceptions import (
     InvalidWindowArgument,
     InvalidAlgorithmArgument,
     InvalidAlgorithmReturnType,
+    InvalidMetadataFieldArgument,
 )
 
 # Regex patterns for validation
@@ -66,24 +67,49 @@ WINDOW_NAME = r"^[A-Z][a-zA-Z0-9]*$"
 LOGGER = logging.getLogger(__name__)
 
 
+@dataclass(frozen=True)
+class MetadataField:
+    name: str
+    description: str
+
+    def __post_init__(self) -> None:
+        if self.name == "":
+            raise InvalidMetadataFieldArgument("Metadata field name cannot be empty")
+
+        if self.description == "":
+            raise InvalidMetadataFieldArgument(
+                "Metadata field description cannot be empty"
+            )
+
+
 @dataclass
 class WindowType:
     name: str
     version: str
     description: str
+    metadataFields: List[MetadataField] = field(default_factory=list)
 
-    def __post__init__(self) -> None:
+    def __post_init__(self) -> None:
         if not re.match(WINDOW_NAME, self.name):
-            raise InvalidAlgorithmArgument(
+            raise InvalidWindowArgument(
                 f"Window name '{self.name}' must be in PascalCase"
             )
 
         if not re.match(SEMVER_PATTERN, self.version):
-            raise InvalidAlgorithmArgument(
+            raise InvalidWindowArgument(
                 f"Window version '{self.version}' must follow basic semantic "
                 "versioning (e.g., '1.0.0') without release portions"
             )
 
+        _seenFields = set()
+        for field in self.metadataFields:
+            if field in _seenFields:
+                raise InvalidWindowArgument(
+                    f"Two or more metadata fields provided with the same name:'{field.name}' and description@ '{field.description}"
+                )
+            else:
+                _seenFields.add(field)
+
 
 @dataclass
 class StructResult:
@@ -213,14 +239,14 @@ def EmitWindow(window: Window) -> None:
     if envs.is_production:
         # secure channel with TLS
         with grpc.secure_channel(
-            envs.ORCASERVER, grpc.ssl_channel_credentials()
+            envs.ORCA_CORE, grpc.ssl_channel_credentials()
         ) as channel:
             stub = service_pb2_grpc.OrcaCoreStub(channel)
             response = stub.EmitWindow(window_pb)
             LOGGER.info(f"Window emitted: {response}")
     else:
         # insecure channel for local development
-        with grpc.insecure_channel(envs.ORCASERVER) as channel:
+        with grpc.insecure_channel(envs.ORCA_CORE) as channel:
             stub = service_pb2_grpc.OrcaCoreStub(channel)
             response = stub.EmitWindow(window_pb)
             LOGGER.info(f"Window emitted: {response}")
@@ -234,8 +260,7 @@ class Algorithm:
     Attributes:
         name (str): The name of the algorithm (PascalCase).
         version (str): Semantic version of the algorithm (e.g., "1.0.0").
-        window_name (str): The window type name that triggers the algorithm.
-        window_version (str): The version of the window type.
+        window_type (WindowType): The window type triggers the algorithm.
         exec_fn (AlgorithmFn): The execution function for the algorithm.
         processor (str): Name of the processor where it's registered.
         runtime (str): Python runtime used for execution.
@@ -243,9 +268,7 @@ class Algorithm:
 
     name: str
     version: str
-    window_name: str
-    window_version: str
-    window_description: str
+    window_type: WindowType
     exec_fn: AlgorithmFn
     processor: str
     runtime: str
@@ -259,7 +282,7 @@ class Algorithm:
     @property
     def full_window_name(self) -> str:
         """Returns the full window name as `window_name_window_version`."""
-        return f"{self.window_name}_{self.window_version}"
+        return f"{self.window_type.name}_{self.window_type.version}"
 
 
 class Algorithms:
@@ -293,7 +316,7 @@ class Algorithms:
             LOGGER.error(f"Attempted to register duplicate algorithm: {name}")
             raise ValueError(f"Algorithm {name} already exists")
         LOGGER.info(
-            f"Registering algorithm: {name} (window: {algorithm.window_name}_{algorithm.window_version})"
+            f"Registering algorithm: {name} (window: {algorithm.window_type.name}_{algorithm.window_type.version})"
         )
         self._algorithms[name] = algorithm
 
@@ -368,10 +391,8 @@ class Processor(OrcaProcessorServicer):  # type: ignore
     def __init__(self, name: str, max_workers: int = 10):
         super().__init__()
         self._name = name
-        self._processorConnStr = (
-            f"0.0.0.0:{envs.PORT}"  # attach the processor to all network interfaces.
-        )
-        self._orcaProcessorConnStr = f"{envs.HOST}:{envs.PORT}"  # tell orca-core to reference this processor by this address.
+        self._processorConnStr = f"[::]:{envs.PROCESSOR_PORT}"  # attach the processor to all network interfaces when launching the gRPC service.
+        self._orcaProcessorConnStr = f"{envs.PROCESSOR_HOST}:{envs.PROCESSOR_EXTERNAL_PORT}"  # tell orca-core to reference this processor by this address.
         self._runtime = sys.version
         self._max_workers = max_workers
         self._algorithmsSingleton: Algorithms = Algorithms()
@@ -639,9 +660,16 @@ class Processor(OrcaProcessorServicer):  # type: ignore
             algo_msg.result_type = result_type_pb
 
             # Add window type
-            algo_msg.window_type.name = algorithm.window_name
-            algo_msg.window_type.version = algorithm.window_version
-            algo_msg.window_type.description = algorithm.window_description
+            algo_msg.window_type.name = algorithm.window_type.name
+            algo_msg.window_type.version = algorithm.window_type.version
+            algo_msg.window_type.description = algorithm.window_type.description
+
+            # Fill in metadata fields if present
+            if len(algorithm.window_type.metadataFields) > 0:
+                for metadataField in algorithm.window_type.metadataFields:
+                    metadata_fields_msg = algo_msg.window_type.metadataFields.add()
+                    metadata_fields_msg.name = metadataField.name
+                    metadata_fields_msg.description = metadataField.description
 
             # Add dependencies if they exist
             if algorithm.full_name in self._algorithmsSingleton._dependencies:
@@ -651,21 +679,29 @@ class Processor(OrcaProcessorServicer):  # type: ignore
                     dep_msg.version = dep.version
                     dep_msg.processor_name = dep.processor
                     dep_msg.processor_runtime = dep.runtime
-
-        if envs.is_production:
-            # secure channel with TLS
-            with grpc.secure_channel(
-                envs.ORCASERVER, grpc.ssl_channel_credentials()
-            ) as channel:
-                stub = service_pb2_grpc.OrcaCoreStub(channel)
-                response = stub.RegisterProcessor(registration_request)
-                LOGGER.info(f"Algorithm registration response received: {response}")
-        else:
-            # insecure channel for local development
-            with grpc.insecure_channel(envs.ORCASERVER) as channel:
-                stub = service_pb2_grpc.OrcaCoreStub(channel)
-                response = stub.RegisterProcessor(registration_request)
-                LOGGER.info(f"Algorithm registration response received: {response}")
+        try:
+            if envs.is_production:
+                # secure channel with TLS
+                with grpc.secure_channel(
+                    envs.ORCA_CORE, grpc.ssl_channel_credentials()
+                ) as channel:
+                    stub = service_pb2_grpc.OrcaCoreStub(channel)
+                    response = stub.RegisterProcessor(registration_request)
+                    LOGGER.info(f"Algorithm registration response received: {response}")
+            else:
+                # insecure channel for local development
+                with grpc.insecure_channel(envs.ORCA_CORE) as channel:
+                    stub = service_pb2_grpc.OrcaCoreStub(channel)
+                    response = stub.RegisterProcessor(registration_request)
+                    LOGGER.info(f"Algorithm registration response received: {response}")
+        except grpc._channel._InactiveRpcError as e:
+            print()
+            print(e.details())
+            sys.exit(1)
+        except Exception as e:
+            print()
+            print(e)
+            sys.exit(1)
 
     def Start(self) -> None:
         """
@@ -703,12 +739,13 @@ class Processor(OrcaProcessorServicer):  # type: ignore
             # add the server port
             port = server.add_insecure_port(self._processorConnStr)
             if port == 0:
-                raise RuntimeError(f"Failed to bind to port {envs.PORT}")
+                raise RuntimeError(f"Failed to bind to port {envs.PROCESSOR_PORT}")
 
             LOGGER.info(f"Server listening on address {self._processorConnStr}")
 
             # start the server
             server.start()
+
             LOGGER.info("Server started successfully")
 
             # setup graceful shutdown
@@ -801,9 +838,7 @@ class Processor(OrcaProcessorServicer):  # type: ignore
             algorithm = Algorithm(
                 name=name,
                 version=version,
-                window_name=window_type.name,
-                window_version=window_type.version,
-                window_description=window_type.description,
+                window_type=window_type,
                 exec_fn=wrapper,
                 processor=self._name,
                 runtime=sys.version,
